from django.shortcuts import render, redirect
from datetime import datetime, date, timedelta
from django.contrib.auth.decorators import login_required, permission_required
from .models import Payment, Fee, FeeRain, Fee100
from users.models import Person
from .filters import PaymentFilter
from dateutil.rrule import rrule, DAILY
import calendar
from django.core.paginator import Paginator
from django.contrib import messages
from management.models import Session, Semester
from decimal import Decimal
import math
from django.utils import timezone

@login_required
def plans(request):
    me = request.user
    level = request.user.level
    session = Session.objects.get(current=True)
    if session:
        time_remaining = session.reg_deadline - timezone.now()
        days = time_remaining.days
        hours = (time_remaining.seconds % 86400) // 3600
        minutes = ((time_remaining.seconds % 86400) % 3600) // 60
        seconds = time_remaining.seconds % 60

        data = {
            'name': "",
            'days': days,
            'hours': hours,
            'minutes': minutes,
            'seconds': seconds
        }
    else:
        data = {
            'name': "",
            'days': 0,
            'hours': 0,
            'minutes': 0,
            'seconds': 0
        }
    programme = request.user.programme
    if me.state == "Benue" or me.state == "Kogi" or me.state == "Kwara" or me.state == "Nasarawa" or me.state == "Niger" or me.state == "Plateau" == True:
        h_bill = programme.amount_nc * Decimal(0.5)
    else:
        h_bill = programme.amount * Decimal(0.5)

    if level.level == 100:
        fees = Fee100.objects.filter(open=True)
    else:
        fees = Fee.objects.filter(open=True)
    h_fees_dict = {}
    for each in fees:
        h_fees_dict[each.type] = each.amount

    h_bills = h_bill
    for each in h_fees_dict.values():
        h_bills = h_bills + each
    harmattan_bill = round(h_bills,2)

    if request.user.state == "Benue" or request.user.state == "Kogi" or request.user.state == "Kwara" or request.user.state == "Nasarawa" or request.user.state == "Niger" or request.user.state == "Plateau":
        r_bill = programme.amount_nc * Decimal(0.5)
    else:
        r_bill = programme.amount * Decimal(0.5)

    r_fees = FeeRain.objects.filter(open=True)
    r_fees_dict = {}
    for each in r_fees:
        r_fees_dict[each.type] = each.amount

    r_bills = r_bill
    for each in r_fees_dict.values():
        r_bills = r_bills + each
    rain_bill = round(r_bills,2)

    h100 = request.user.h100
    r100 = request.user.r100
    h200 = request.user.h200
    r200 = request.user.r200
    h300 = request.user.h300
    r300 = request.user.r300
    h400 = request.user.h400
    r400 = request.user.r400

    if level.level == 100:
        if h100 == True:
            okay_1 = "Yes"
        else:
            okay_1 = "No"
        if r100 == True:
            okay_2 = "Yes"
        else:
            okay_2 = "No"
    elif level.level == 200:
        if h200 == True:
            okay_1 = "Yes"
        else:
            okay_1 = "No"
        if r200 == True:
            okay_2 = "Yes"
        else:
            okay_2 = "No"
    elif level.level == 300:
        if h300 == True:
            okay_1 = "Yes"
        else:
            okay_1 = "No"
        if r300 == True:
            okay_2 = "Yes"
        else:
            okay_2 = "No"
    elif level.level == 400:
        if h400 == True:
            okay_1 = "Yes"
        else:
            okay_1 = "No"
        if r400 == True:
            okay_2 = "Yes"
        else:
            okay_2 = "No"

    context = {'harmattan_bill':harmattan_bill, 'rain_bill':rain_bill, 'okay_1':okay_1, 'okay_2':okay_2,
    'data': data, 'time_remaining':time_remaining}
    return render(request, 'payment/plans.html', context)

@login_required
def upgrade(request):
    key = 'pk_test_8a16050ea1f660e4812568a66b5299cda27add91'
    level = request.user.level
    session = Session.objects.get(current=True)

    programme = request.user.programme
    if request.user.state == "Benue" or request.user.state == "Kogi" or request.user.state == "Kwara" or request.user.state == "Nasarawa" or request.user.state == "Niger" or request.user.state == "Plateau":
        h_bill = programme.amount_nc * Decimal(0.5)
    else:
        h_bill = programme.amount * Decimal(0.5)

    h_bill = round(h_bill,2)
    if level.level == 100:
        fees = Fee100.objects.filter(open=True)
    else:
        fees = Fee.objects.filter(open=True)

    h_fees_dict = {}
    for each in fees:
        h_fees_dict[each.type] = each.amount
    h_bills = h_bill
    for each in h_fees_dict.values():
        h_bills = h_bills + each
    harmattan_bill = round(h_bills,2)

    h100 = request.user.h100
    h200 = request.user.h200
    h300 = request.user.h300
    h400 = request.user.h400

    if level.level == 100:
        if h100 == True:
            okay_1 = "Yes"
        else:
            okay_1 = "No"
    elif level.level == 200:
        if h200 == True:
            okay_1 = "Yes"
        else:
            okay_1 = "No"
    elif level.level == 300:
        if h300 == True:
            okay_1 = "Yes"
        else:
            okay_1 = "No"
    elif level.level == 400:
        if h400 == True:
            okay_1 = "Yes"
        else:
            okay_1 = "No"

    context = {'harmattan_bill':harmattan_bill, 'h_bill':h_bill,
    'okay_1':okay_1, 'session':session, 'key':key, 'h_fees_dict':h_fees_dict}

    return render(request, 'payment/harmattan_payment.html', context)

@login_required
def upgradeRain(request):
    key = 'pk_test_8a16050ea1f660e4812568a66b5299cda27add91'
    level = request.user.level
    session = Session.objects.get(current=True)

    programme = request.user.programme

    # if level.level == 100:
    #     fees = FeeRain.objects.filter(open=True)
    # else:
    #     fees = FeeRain.objects.filter(open=True)
    fees = FeeRain.objects.filter(open=True)

    r_fees_dict = {}
    for each in fees:
        r_fees_dict[each.type] = each.amount

    if request.user.state == "Benue" or request.user.state == "Kogi" or request.user.state == "Kwara" or request.user.state == "Nasarawa" or request.user.state == "Niger" or request.user.state == "Plateau":
        r_bill = programme.amount_nc * Decimal(0.5)
    else:
        r_bill = programme.amount * Decimal(0.5)
    r_bill = round(r_bill,2)

    r_bills = r_bill
    for each in r_fees_dict.values():
        r_bills = r_bills + each
    rain_bill = round(r_bills,2)

    r100 = request.user.r100
    r200 = request.user.r200
    r300 = request.user.r300
    r400 = request.user.r400

    if level.level == 100:
        if r100 == True:
            okay_1 = "Yes"
        else:
            okay_1 = "No"
    elif level.level == 200:
        if r200 == True:
            okay_1 = "Yes"
        else:
            okay_1 = "No"
    elif level.level == 300:
        if r300 == True:
            okay_1 = "Yes"
        else:
            okay_1 = "No"
    elif level.level == 400:
        if r400 == True:
            okay_1 = "Yes"
        else:
            okay_1 = "No"

    context = {'rain_bill':rain_bill, 'r_bill':r_bill,
    'okay_1':okay_1, 'session':session, 'key':key, 'r_fees_dict':r_fees_dict}
    return render(request, 'payment/rain_payment.html', context)

@login_required
def harmattan(request):
    level = request.user.level
    session = Session.objects.get(current=True)

    programme = request.user.programme
    if request.user.state == "Benue" or request.user.state == "Kogi" or request.user.state == "Kwara" or request.user.state == "Nasarawa" or request.user.state == "Niger" or request.user.state == "Plateau":
        h_bill = programme.amount_nc * Decimal(0.5)
    else:
        h_bill = programme.amount * Decimal(0.5)

    if level.level == 100:
        fees = Fee100.objects.filter(open=True)
    else:
        fees = Fee.objects.filter(open=True)
    fees_dict = {}
    for each in fees:
        fees_dict[each.type] = each.amount

    h_bills = h_bill
    for each in fees_dict.values():
        h_bills = h_bills + each
    harmattan_bill = round(h_bills,2)

    x = datetime.now().year + datetime.now().month + datetime.now().day + datetime.now().hour + datetime.now().minute + datetime.now().second + 99999*datetime.now().microsecond
    y = "H" + str(x)

    new_entry = Payment()
    new_entry.session = session
    new_entry.semester = "Harmattan"
    new_entry.payment_id = y
    new_entry.user = request.user
    new_entry.static_level = request.user.level
    new_entry.amount_paid = harmattan_bill
    new_entry.save()

    me = Person.objects.get(username=request.user.username)

    if level.level == 100:
        me.h100 = True
        me.save()
    elif level.level == 200:
        me.h200 = True
        me.save()
    elif level.level == 300:
        me.h300 = True
        me.save()
    elif level.level == 400:
        me.h400 = True
        me.save()

    messages.success(request, "Thank you! Your payment was successful. You can print your receipt below")
    return redirect('history')

@login_required
def rain(request):
    level = request.user.level
    session = Session.objects.get(current=True)

    programme = request.user.programme
    if request.user.state == "Benue" or request.user.state == "Kogi" or request.user.state == "Kwara" or request.user.state == "Nasarawa" or request.user.state == "Niger" or request.user.state == "Plateau":
        r_bill = programme.amount_nc * Decimal(0.5)
    else:
        r_bill = programme.amount * Decimal(0.5)

    # if level.level == 100:
    #     fees = Fee100.objects.filter(open=True)
    # else:
    #     fees = Fee.objects.filter(open=True)
    fees = FeeRain.objects.filter(open=True)

    fees_dict = {}
    for each in fees:
        fees_dict[each.type] = each.amount

    r_bills = r_bill
    for each in fees_dict.values():
        r_bills = r_bills + each
    rain_bill = round(r_bills,2)

    x = datetime.now().year + datetime.now().month + datetime.now().day + datetime.now().hour + datetime.now().minute + datetime.now().second + 99999*datetime.now().microsecond
    y = "R" + str(x)

    new_entry = Payment()
    new_entry.session = session
    new_entry.semester = "Rain"
    new_entry.payment_id = y
    new_entry.user = request.user
    new_entry.static_level = request.user.level
    new_entry.amount_paid = rain_bill
    new_entry.save()

    me = Person.objects.get(username=request.user.username)

    if level.level == 100:
        me.r100 = True
        me.save()
    elif level.level == 200:
        me.r200 = True
        me.save()
    elif level.level == 300:
        me.r300 = True
        me.save()
    elif level.level == 400:
        me.r400 = True
        me.save()

    messages.success(request, "Thank you! Your payment was successful. You can print your receipt below")
    return redirect('history')

@login_required
def history(request):
    context = {}
    filtered_payments = PaymentFilter(
        request.GET,
        queryset = Payment.objects.filter(user=request.user)
    )
    context['filtered_payments'] = filtered_payments
    paginated_filtered_payments = Paginator(filtered_payments.qs, 12)
    page_number = request.GET.get('page')
    payments_page_obj = paginated_filtered_payments.get_page(page_number)
    context['payments_page_obj'] = payments_page_obj
    total_payments = filtered_payments.qs.count()
    context['total_payments'] = total_payments
    return render(request, 'payment/history.html', context)

@login_required
def receipt(request, payment_id, **kwargs):
    level = request.user.level
    payment = Payment.objects.get(payment_id=payment_id)
    if payment.user == request.user:
        programme = request.user.programme
        if request.user.state == "Benue" or request.user.state == "Kogi" or request.user.state == "Kwara" or request.user.state == "Nasarawa" or request.user.state == "Niger" or request.user.state == "Plateau":
            h_bill = programme.amount_nc * Decimal(0.5)
        else:
            h_bill = programme.amount * Decimal(0.5)

        h_bill = round(h_bill,2)
        if level.level == 100:
            fees = Fee100.objects.filter(open=True)
        else:
            fees = Fee.objects.filter(open=True)

        h_fees_dict = {}
        for each in fees:
            h_fees_dict[each.type] = each.amount
        h_bills = h_bill
        for each in h_fees_dict.values():
            h_bills = h_bills + each
        harmattan_bill = round(h_bills,2)

        if request.user.state == "Benue" or request.user.state == "Kogi" or request.user.state == "Kwara" or request.user.state == "Nasarawa" or request.user.state == "Niger" or request.user.state == "Plateau":
            r_bill = programme.amount_nc * Decimal(0.5)
        else:
            r_bill = programme.amount * Decimal(0.5)

        r_bill = round(r_bill,2)
        r_fees = FeeRain.objects.filter(open=True)

        r_fees_dict = {}
        for each in r_fees:
            r_fees_dict[each.type] = each.amount

        r_bills = r_bill
        for each in r_fees_dict.values():
            r_bills = r_bills + each
        rain_bill = round(r_bills,2)

        context = {'payment': payment, 'harmattan_bill':harmattan_bill, 'h_bill':h_bill, 'h_fees_dict':h_fees_dict, 'r_fees_dict':r_fees_dict}
        return render(request, 'payment/receipt.html', context)
    else:
        return redirect('login')
